#include "stm32f4xx_conf.h"
#include "delayex.h"

static volatile uint16_t _cnt;

void delayex_init()
{
	/* APB1 = 168MHz / 4 = 42MHz */
	/* APB2 = 168MHz / 2 = 84MHz */

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);

	TIM_TimeBaseInitTypeDef tis;
	TIM_TimeBaseStructInit(&tis);
	tis.TIM_Prescaler = 420 - 1; 	// 42Mhz  / 420 = 100kHz
	tis.TIM_Period    = 200 - 1; 	// 100kHz / 200 = 0.5kHz = 0.002s = 2ms
	TIM_TimeBaseInit(TIM2, &tis);

	NVIC_InitTypeDef nis;
	nis.NVIC_IRQChannel = TIM2_IRQn;
	nis.NVIC_IRQChannelPreemptionPriority = 0x00;
	nis.NVIC_IRQChannelSubPriority = 0x01;
	nis.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&nis);

	TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);
	TIM_Cmd(TIM2, ENABLE);
}

void delayex_deinit()
{
	TIM_Cmd(TIM2, DISABLE);
	TIM_ITConfig(TIM2, TIM_IT_Update, DISABLE);
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, DISABLE);
}

void delayex_ms(const uint16_t ms)
{
	TIM_ITConfig(TIM2, TIM_IT_Update, DISABLE);
	_cnt = 0;
	TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);

	while(_cnt < ms);
}

extern  void TIM2_IRQHandler(void)
{
	TIM_ClearITPendingBit(TIM2, TIM_IT_Update);
	_cnt++;
}
