#include <stdlib.h>
#include "ssd1289.h"
#include "ssd1289_config.h"
#include "gfx.h"

#define GFXSWAP(x, y) do{x ^= y; y ^= x; x ^= y;} while(0)

/* accessed in ssd1289.c */
uint16_t _gfx_color = WHITE;
uint16_t _txt_bg_color = BLACK;
uint16_t _txt_fg_color = WHITE;
uint16_t _txt_crsr_x = 0;
uint16_t _txt_crsr_y = 0;

static const GUI_FONT* _cur_font = 0;

void gfx_init()
{
	ssd1289_init();
}

const uint16_t gfx_get_max_x()
{
	return GetMaxX();
}

const uint16_t gfx_get_max_y()
{
	return GetMaxY();
}

void gfx_set_color(const uint16_t clr)
{
	_gfx_color = clr;
}

const uint16_t gfx_get_color()
{
	return _gfx_color;
}

void gfx_set_txt_bg_color(const uint16_t clr)
{
	_txt_bg_color = clr;
}

const uint16_t gfx_get_txt_bg_color()
{
	return _txt_bg_color;
}

void gfx_set_txt_fg_color(const uint16_t clr)
{
	_txt_fg_color = clr;
}

const uint16_t gfx_get_txt_fg_color()
{
	return _txt_fg_color;
}

void gfx_set_txt_crsr(const uint16_t x, const uint16_t y)
{
	_txt_crsr_x = x;
	_txt_crsr_y = y;
}

void gfx_get_txt_crsr(uint16_t* x, uint16_t* y)
{
	*x = _txt_crsr_x;
	*y =_txt_crsr_y;
}

void gfx_clear(const uint16_t clr)
{
	ssd1289_clear(clr);
}

void gfx_put_pixel(const uint16_t x, const uint16_t y)
{
	ssd1289_put_pixel(x, y);
}

void gfx_put_pixel_color(const uint16_t x, const uint16_t y, const uint16_t clr)
{
	ssd1289_put_pixel_color(x, y, clr);
}

void gfx_rect(uint16_t left, uint16_t top, uint16_t right, uint16_t bottom)
{
	if(left > right)
		GFXSWAP(left, right);

	if(top > bottom)
		GFXSWAP(top, bottom);

	ssd1289_bar(left,  top,    right, top,    _gfx_color);	// top
	ssd1289_bar(left,  bottom, right, bottom, _gfx_color);	// bottom
	ssd1289_bar(left,  top,    left,  bottom, _gfx_color);	// left
	ssd1289_bar(right, top,    right, bottom, _gfx_color);	// right
}

void gfx_rect_filled(uint16_t left, uint16_t top, uint16_t right, uint16_t bottom)
{
	if(left > right)
		GFXSWAP(left, right);

	if(top > bottom)
		GFXSWAP(top, bottom);

	ssd1289_bar(left, top, right, bottom, _gfx_color);
}

void gfx_line(const uint16_t left, const uint16_t top, const uint16_t right, const uint16_t bottom)
{
	int16_t x, y, addx, addy, dx, dy;
	int32_t p;

	/* vertical / horizontal line */
	if ((left == right) || (top == bottom))
	{
		gfx_rect_filled(left, top, right, bottom);
		return;
	}

	dx = abs((int16_t)(right - left));
	dy = abs((int16_t)(bottom - top));

	x = left;
	y = top;

	addx = addy = 1;

	if(left > right)
		addx = -1;

	if(top > bottom)
		addy = -1;

	if(dx >= dy)
	{
		p = 2L * dy - dx;
		for(int16_t i = 0; i <= dx; ++i)
		{
			gfx_put_pixel(x, y);

			if(p < 0)
	  		{
	    		p += 2 * dy;
	    		x += addx;
	  		}
			else
			{
	    		p += 2 * dy - 2*dx;
	    		x += addx;
	    		y += addy;
	  		}
		}
	}
	else
	{
		p = 2L * dx - dy;

		for(int16_t i = 0; i <= dy; ++i)
		{
			gfx_put_pixel(x, y);
	  		if(p < 0)
	  		{
	    		p += 2 * dx;
	    		y += addy;
	  		}
	  		else
	  		{
	    		p += 2 * dx - 2 * dy;
	    		x += addx;
	    		y += addy;
	  		}
		}
	}
}

/* ref: miniSTM32_lcd.c */
void gfx_circle(const uint16_t x, const uint16_t y, const uint16_t r)
{
    int16_t dx = 0;
    int16_t dy = r;
    int16_t p = 1 - r;

	while(dx < dy)
	{
		gfx_put_pixel(x + dx, y + dy);
		gfx_put_pixel(x - dx, y + dy);
		gfx_put_pixel(x + dx, y - dy);
		gfx_put_pixel(x - dx, y - dy);

		gfx_put_pixel(x + dy, y + dx);
		gfx_put_pixel(x - dy, y + dx);
		gfx_put_pixel(x + dy, y - dx);
		gfx_put_pixel(x - dy, y - dx);

		dx++;

		if(p < 0)
			p = p + (dx << 1) + 1;
		else
		{
			dy--;
			p = p + ((dx - dy) << 1) + 1;
		}

		gfx_put_pixel(x + dx, y + dy);
		gfx_put_pixel(x - dx, y + dy);
		gfx_put_pixel(x + dx, y - dy);
		gfx_put_pixel(x - dx, y - dy);

		gfx_put_pixel(x + dy, y + dx);
		gfx_put_pixel(x - dy, y + dx);
		gfx_put_pixel(x + dy, y - dx);
		gfx_put_pixel(x - dy, y - dx);
	}
}

/* ref: miniSTM32_lcd.c */
void gfx_circle_filled(const uint16_t x, const uint16_t y, const uint16_t r)
{
	int16_t dx = 0;
	int16_t dy = r;
	int16_t p = 1 - r;

	while(dx < dy)
	{
		/* vertical filling */
		gfx_rect_filled(x + dx, y - dy, x + dx, y + dy);	/* from 1th to 4th octant */
		gfx_rect_filled(x - dx, y - dy, x - dx, y + dy);	/* from 8th to 5th octant */
		gfx_rect_filled(x + dy, y - dx, x + dy, y + dx);	/* from 2th to 3th octant */
		gfx_rect_filled(x - dy, y - dx, x - dy, y + dx);	/* from 7th to 6th octant */

		dx++;

		if(p < 0)
			p = p + (dx << 1) + 1;
		else
		{
			dy--;
			p = p + ((dx - dy) << 1) + 1;
		}

		gfx_rect_filled(x + dx, y - dy, x + dx, y + dy);
		gfx_rect_filled(x - dx, y - dy, x - dx, y + dy);
		gfx_rect_filled(x + dy, y - dx, x + dy, y + dx);
		gfx_rect_filled(x - dy, y - dx, x - dy, y + dx);
	}
}

/* ref: miniSTM32_lcd.c */
void gfx_ellipse(const uint16_t x, const uint16_t y, const uint16_t rx, const uint16_t ry)
{
	int32_t sx = rx * rx;
	int32_t sy = ry * ry;

	int32_t dx = 0, dy = ry, p;
	int32_t px = 0, py = (sx * dy) << 1;

	gfx_put_pixel(x + dx, y + dy);	/* 2nd quadrant */
	gfx_put_pixel(x - dx, y + dy);	/* 3rd quadrant */
	gfx_put_pixel(x + dx, y - dy);	/* 1st quadrant */
	gfx_put_pixel(x - dx, y - dy);	/* 4th quadrant */

	/* region 1 */
	p = sy - (sx * ry) + (sx >> 2);

	while(px < py)
	{
		dx++;
		px = px + (sy << 1);

		if(p < 0)
			p = p + sy + px;
		else
		{
			dy--;
			py = py - (sx << 1);
			p = p + sy + px - py;
		}

		gfx_put_pixel(x + dx, y + dy);
		gfx_put_pixel(x - dx, y + dy);
		gfx_put_pixel(x + dx, y - dy);
		gfx_put_pixel(x - dx, y - dy);
	}

	/* region 2 */
	p = (sy * ((dx << 1) + 1) * ((dx << 1) + 1) + sx * ((dy << 1) - 2) * ((dy << 1) - 2) - ((sx * sy) << 2)) >> 2;

	while(dy > 0)
	{
		dy--;
		py = py - (sx << 1);

		if(p > 0)
			p = p + sx - py;
		else
		{
			dx++;
			px = px + (sy << 1);
			p = p + sx - py + px;
		}

		gfx_put_pixel(x + dx, y + dy);
		gfx_put_pixel(x - dx, y + dy);
		gfx_put_pixel(x + dx, y - dy);
		gfx_put_pixel(x - dx, y - dy);
	}
}

/* ref: miniSTM32_lcd.c */
void gfx_ellipse_filled(const uint16_t x, const uint16_t y, const uint16_t rx, const uint16_t ry)
{
	int32_t sx = rx * rx;
	int32_t sy = ry * ry;

	int32_t dx = 0, dy = ry, p;
	int32_t px = 0, py = (sx * dy) << 1;

	/* vertical filling */
	gfx_rect_filled(x + dx, y - dy, x + dx, y + dy);	/* from 1st to 2nd quadrant */
	gfx_rect_filled(x - dx, y - dy, x - dx, y + dy);	/* from 4th to 3rd quadrant */

	/* region 1 */
	p = sy - (sx * ry) + (sx>>2);

	while(px < py)
	{
		dx++;
		px = px + (sy << 1);

		if(p < 0)
			p = p + sy + px;
		else
		{
			dy--;
			py = py - (sx << 1);
			p = p + sy + px - py;
		}

		gfx_rect_filled(x + dx, y - dy, x + dx, y + dy);
		gfx_rect_filled(x - dx, y - dy, x - dx, y + dy);
	}

	/* region 2 */
	p = (sy * ((dx << 1) + 1) * ((dx << 1) + 1) + sx * ((dy << 1) - 2) * ((dy << 1) - 2) - ((sx * sy) << 2)) >> 2;

	while(dy > 0)
	{
		dy--;
		py = py - (sx<<1);

		if(p > 0)
			p = p + sx - py;
		else
		{
			dx++;
			px = px + (sy << 1);
			p = p + sx - py + px;
		}

		gfx_rect_filled(x + dx, y - dy, x + dx, y + dy);
		gfx_rect_filled(x - dx, y - dy, x - dx, y + dy);
	}
}

void gfx_set_font(const GUI_FONT* ft)
{
	_cur_font = ft;
}

void gfx_put_char(const char c)
{
	if(! _cur_font)
		return;

	const GUI_FONT_PROP* fp = _cur_font->pFontData;

	if(c == '\n')
	{
		_txt_crsr_y += _cur_font->YSize;
		_txt_crsr_x = 0;
		return;
	}

	for(; fp; fp = fp->pNext)
	{
		if((c >= fp->First) && (c <= fp->Last))
			break;
	}

	if(!fp)
		return;

	const GUI_CHARINFO* ci = fp->paCharInfo + (c - fp->First);

	ssd1289_draw_bitmap_mono(ci->XSize, _cur_font->YSize, ci->pData);

	_txt_crsr_x += ci->XSize + 1;
}

void gfx_put_string(const char* s)
{
	while(*s)
		gfx_put_char(*s++);
}

void gfx_draw_pic(const uint16_t left, const uint16_t top, const uint16_t right, const uint16_t bottom, const uint16_t* data)
{
	ssd1289_draw_pic(left, top, right, bottom, data);
}
