
#ifndef _GFX_H_
#define _GFX_H_

#include <stdint.h>
#include <stdbool.h>
#include "font.h"

/* RGB565 */
#define RGB(r, g, b) ((uint16_t)((((r) & 0xf8) << 8) | (((g) & 0xfc)<<3) | (((b) & 0xf8) >> 3)))

#define BLACK   	RGB(0x00, 0x00, 0x00)
#define WHITE   	RGB(0xff, 0xff, 0xff)
#define RED     	RGB(0xff, 0x00, 0x00)
#define GREEN   	RGB(0x00, 0xff, 0x00)
#define BLUE    	RGB(0x00, 0x00, 0xff)
#define YELLOW  	RGB(0xff, 0xff, 0x00)
#define MAGENTA		RGB(0xff, 0x00, 0xff)
#define CYAN    	RGB(0x00, 0xff, 0xff)
#define GRAY    	RGB(0x80, 0x80, 0x80)
#define SILVER  	RGB(0xa0, 0xa0, 0x80)
#define GOLD    	RGB(0xa0, 0xa0, 0x40)
#define LBLUE 		RGB(0x60, 0x60, 0xff)
#define LRED    	RGB(0xff, 0x60, 0x60)


void gfx_init();

const uint16_t gfx_get_max_x();
const uint16_t gfx_get_max_y();

void gfx_set_color(const uint16_t clr);
const uint16_t gfx_get_color();

void gfx_clear(const uint16_t clr);
void gfx_put_pixel(const uint16_t x, const uint16_t y);
void gfx_put_pixel_color(const uint16_t x, const uint16_t y, const uint16_t clr);

void gfx_rect(uint16_t left, uint16_t top, uint16_t right, uint16_t bottom);
void gfx_rect_filled(uint16_t left, uint16_t top, uint16_t right, uint16_t bottom);

void gfx_line(const uint16_t left, const uint16_t top, const uint16_t right, const uint16_t bottom);

void gfx_circle(const uint16_t x, const uint16_t y, const uint16_t r);
void gfx_circle_filled(const uint16_t x, const uint16_t y, const uint16_t r);

void gfx_ellipse(const uint16_t x, const uint16_t y, const uint16_t rx, const uint16_t ry);
void gfx_ellipse_filled(const uint16_t x, const uint16_t y, const uint16_t rx, const uint16_t ry);

void gfx_set_txt_bg_color(const uint16_t clr);
const uint16_t gfx_get_txt_bg_color();

void gfx_set_txt_fg_color(const uint16_t clr);
const uint16_t gfx_get_txt_fg_color();

void gfx_set_txt_crsr(const uint16_t x, const uint16_t y);
void gfx_get_txt_crsr(uint16_t* x, uint16_t* y);

void gfx_set_font(const GUI_FONT* ft);
void gfx_put_char(const char c);
void gfx_put_string(const char* s);


void gfx_draw_pic(const uint16_t left, const uint16_t top, const uint16_t right, const uint16_t bottom, const uint16_t* data);


#endif // _GFX_H_
