#include "stm32f4xx_conf.h"
#include "rtc.h"

#define RTC_CLOCK_SOURCE_LSE
//#define RTC_CLOCK_SOURCE_LSI

const char* rtc_dow_txt[] =
{
	"Error",
	"Montag", "Dienstag", "Mittwoch",
	"Donnerstag", "Freitag", "Samstag",
	"Sonntag"
};

const char* rtc_month_txt[] =
{
	"Error",
	"Januar", "Februar", "Mrz",
	"April", "Mai", "Juni",
	"Juli",	"August", "September",
	"Oktober", "November", "Dezember"
};


void rtc_init()
{
	/* Enable the PWR APB1 Clock Interface */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);

	/* Allow access to BKP Domain */
	PWR_BackupAccessCmd(ENABLE);

	if(RTC_ReadBackupRegister(RTC_BKP_DR1) != 0xa5a5)
	{
		// test
		PWR_PVDCmd(ENABLE);
		PWR_PVDLevelConfig(PWR_PVDLevel_1);

#if defined (RTC_CLOCK_SOURCE_LSI) /* LSI used as RTC source clock*/

		/* The RTC Clock may varies due to LSI frequency dispersion. */
		/* Enable the LSI OSC */
		RCC_LSICmd(ENABLE);

		/* Wait till LSI is ready */
		while(RCC_GetFlagStatus(RCC_FLAG_LSIRDY) == RESET)
		{
		}

		/* Select the RTC Clock Source */
		RCC_RTCCLKConfig(RCC_RTCCLKSource_LSI);

#elif defined (RTC_CLOCK_SOURCE_LSE) /* LSE used as RTC source clock */

		/* Enable the LSE OSC */
		RCC_LSEConfig(RCC_LSE_ON);

		/* Wait till LSE is ready */
		while(RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET)
		{
		}

		/* Select the RTC Clock Source */
		RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);

#else
	#error Please select the RTC Clock source inside the main.c file
#endif

		/* Enable the RTC Clock */
		RCC_RTCCLKCmd(ENABLE);

		/* Wait for RTC APB registers synchronisation */
		RTC_WaitForSynchro();

		/* Allow access to BKP Domain */
		PWR_BackupAccessCmd(ENABLE);

		/* Write to the first RTC Backup Data Register */
		RTC_WriteBackupRegister(RTC_BKP_DR1, 0xa5a5);

		/* Calendar Configuration */
		RTC_InitTypeDef ri;
		ri.RTC_AsynchPrediv = 0x7f;
		ri.RTC_SynchPrediv 	= 0xff;
		ri.RTC_HourFormat 	= RTC_HourFormat_24;
		RTC_Init(&ri);

		/* Set the Time to default */
		RTC_TimeTypeDef rt;
		rt.RTC_Hours 	= 0;
		rt.RTC_Minutes 	= 0;
		rt.RTC_Seconds 	= 0;
		RTC_SetTime(RTC_Format_BIN, &rt);

		/* Set the Date to default */
		RTC_DateTypeDef rd;
		rd.RTC_Month 	= 1;
		rd.RTC_Date 	= 1;
		rd.RTC_Year 	= 0;
		rd.RTC_WeekDay  = RTC_Weekday_Saturday;
		RTC_SetDate(RTC_Format_BIN, &rd);

		/* Enable BKPRAM Clock */
		RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_BKPSRAM, ENABLE);

		/* Enable the Backup SRAM low power Regulator to retain it's content in VBAT mode */
		PWR_BackupRegulatorCmd(ENABLE);

		/* Wait until the Backup SRAM low power Regulator is ready */
		while(PWR_GetFlagStatus(PWR_FLAG_BRR) == RESET)
		{
		}
	}
	else
	{
		/* Wait for RTC APB registers synchronisation */
		RTC_WaitForSynchro();

		/* Clear the Wakeup Interrupt */
		RTC_ClearITPendingBit(RTC_IT_WUT);

		/* Enable BKPSRAM Clock */
		RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_BKPSRAM, ENABLE);
	}
}

const char* rtc_get_dow_str(uint8_t dow)
{
	if(dow > (sizeof(rtc_dow_txt) / sizeof(rtc_dow_txt[0])))
		return rtc_dow_txt[0];	// Error

	return rtc_dow_txt[dow];
};

const char* rtc_get_month_str(uint8_t month)
{
	if(month > (sizeof(rtc_month_txt) / sizeof(rtc_month_txt[0])))
		return rtc_month_txt[0];	// Error

	return rtc_month_txt[month];
};
