#include <string.h>
#include "stm32f4xx_conf.h"
#include "xmem.h"

/* SRAM: IS62WV51216BLL, 512K x 16 */
#define XMEM_START 0x64000000
#define XMEM_END   0x640fffff	// = 1MB

#define XMEM_SIZE_OF_PAGE       512  // 512 Bytes pages
#define XMEM_MAX_PAGE_NUMBERS  2048  // Handle a maximum of 2048 pages (2048 * 512 = 1.048.576Byte)

void xmem_init()
{
	/* Enable GPIOs clock */
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD |
			               RCC_AHB1Periph_GPIOE |
			               RCC_AHB1Periph_GPIOF |
			               RCC_AHB1Periph_GPIOG,
			               ENABLE);

	/* Enable FSMC clock */
	RCC_AHB3PeriphClockCmd(RCC_AHB3Periph_FSMC, ENABLE);

	/*
	 +------------------------------------------------------------------------------+
	 | GPIOS                                                                        |
	 +-------------------+--------------------+------------------+------------------+
	 | SRAM pins assignment                                                         |
	 +-------------------+--------------------+------------------+------------------+
	 | PD0  <-> FSMC_D2  | PE0  <-> FSMC_NBL0 | PF0  <-> FSMC_A0 | PG0 <-> FSMC_A10 |
	 | PD1  <-> FSMC_D3  | PE1  <-> FSMC_NBL1 | PF1  <-> FSMC_A1 | PG1 <-> FSMC_A11 |
	 | PD4  <-> FSMC_NOE | PE7  <-> FSMC_D4   | PF2  <-> FSMC_A2 | PG2 <-> FSMC_A12 |
	 | PD5  <-> FSMC_NWE | PE8  <-> FSMC_D5   | PF3  <-> FSMC_A3 | PG3 <-> FSMC_A13 |
	 | PD8  <-> FSMC_D13 | PE9  <-> FSMC_D6   | PF4  <-> FSMC_A4 | PG4 <-> FSMC_A14 |
	 | PD9  <-> FSMC_D14 | PE10 <-> FSMC_D7   | PF5  <-> FSMC_A5 | PG5 <-> FSMC_A15 |
	 | PD10 <-> FSMC_D15 | PE11 <-> FSMC_D8   | PF12 <-> FSMC_A6 | PG9 <-> FSMC_NE2 |
	 | PD11 <-> FSMC_A16 | PE12 <-> FSMC_D9   | PF13 <-> FSMC_A7 |------------------+
	 | PD12 <-> FSMC_A17 | PE13 <-> FSMC_D10  | PF14 <-> FSMC_A8 |
	 | PD13 <-> FSMC_A18 | PE14 <-> FSMC_D11  | PF15 <-> FSMC_A9 |
	 | PD14 <-> FSMC_D0  | PE15 <-> FSMC_D12  |------------------+
	 | PD15 <-> FSMC_D1  |--------------------+
	 +-------------------+
	*/

	/* GPIOD configuration */
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource0, GPIO_AF_FSMC);		// D2
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource1, GPIO_AF_FSMC);		// D3
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource4, GPIO_AF_FSMC);		// OE
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource5, GPIO_AF_FSMC);		// WE
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource8, GPIO_AF_FSMC);		// D13
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource9, GPIO_AF_FSMC);		// D14
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource10, GPIO_AF_FSMC);	// D15
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource11, GPIO_AF_FSMC);	// A16
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource12, GPIO_AF_FSMC);	// A17
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource13, GPIO_AF_FSMC);	// A18
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource14, GPIO_AF_FSMC);	// D0
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource15, GPIO_AF_FSMC);	// D1

	GPIO_InitTypeDef gpi;
	gpi.GPIO_Pin = GPIO_Pin_0  | GPIO_Pin_1  | GPIO_Pin_4  |
			       GPIO_Pin_5  | GPIO_Pin_8  | GPIO_Pin_9  |
			       GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 |
			       GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
	gpi.GPIO_Mode  = GPIO_Mode_AF;
	gpi.GPIO_Speed = GPIO_Speed_100MHz;
	gpi.GPIO_OType = GPIO_OType_PP;
	gpi.GPIO_PuPd  = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOD, &gpi);

	/* GPIOE configuration */
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource0 , GPIO_AF_FSMC);	// LB
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource1 , GPIO_AF_FSMC);	// UB
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource7 , GPIO_AF_FSMC);	// D4
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource8 , GPIO_AF_FSMC);	// D5
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource9 , GPIO_AF_FSMC);	// D6
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource10 , GPIO_AF_FSMC);	// D7
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource11 , GPIO_AF_FSMC);	// D8
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource12 , GPIO_AF_FSMC);	// D9
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource13 , GPIO_AF_FSMC);	// D10
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource14 , GPIO_AF_FSMC);	// D11
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource15 , GPIO_AF_FSMC);	// D12

	gpi.GPIO_Pin = GPIO_Pin_0  | GPIO_Pin_1  | GPIO_Pin_7  |
			       GPIO_Pin_8  | GPIO_Pin_9  | GPIO_Pin_10 |
			       GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 |
			       GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_Init(GPIOE, &gpi);

	/* GPIOF configuration */
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource0 , GPIO_AF_FSMC);	// A0
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource1 , GPIO_AF_FSMC);	// A1
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource2 , GPIO_AF_FSMC);	// A2
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource3 , GPIO_AF_FSMC);	// A3
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource4 , GPIO_AF_FSMC);	// A4
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource5 , GPIO_AF_FSMC);	// A5
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource12 , GPIO_AF_FSMC);	// A6
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource13 , GPIO_AF_FSMC);	// A7
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource14 , GPIO_AF_FSMC);	// A8
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource15 , GPIO_AF_FSMC);	// A9

	gpi.GPIO_Pin = GPIO_Pin_0  | GPIO_Pin_1  | GPIO_Pin_2  |
			       GPIO_Pin_3  | GPIO_Pin_4  | GPIO_Pin_5  |
			       GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 |
			       GPIO_Pin_15;
	GPIO_Init(GPIOF, &gpi);

	/* GPIOG configuration */
	GPIO_PinAFConfig(GPIOG, GPIO_PinSource0 , GPIO_AF_FSMC);	// A10
	GPIO_PinAFConfig(GPIOG, GPIO_PinSource1 , GPIO_AF_FSMC);	// A11
	GPIO_PinAFConfig(GPIOG, GPIO_PinSource2 , GPIO_AF_FSMC);	// A12
	GPIO_PinAFConfig(GPIOG, GPIO_PinSource3 , GPIO_AF_FSMC);	// A13
	GPIO_PinAFConfig(GPIOG, GPIO_PinSource4 , GPIO_AF_FSMC);	// A14
	GPIO_PinAFConfig(GPIOG, GPIO_PinSource5 , GPIO_AF_FSMC);	// A15
	GPIO_PinAFConfig(GPIOG, GPIO_PinSource9 , GPIO_AF_FSMC);	// CE
	gpi.GPIO_Pin = GPIO_Pin_0  | GPIO_Pin_1  | GPIO_Pin_2  |
			       GPIO_Pin_3  | GPIO_Pin_4  | GPIO_Pin_5  |
			       GPIO_Pin_9;
	GPIO_Init(GPIOG, &gpi);

	/* FSMC Configuration */
	FSMC_NORSRAMTimingInitTypeDef nst;

	nst.FSMC_AddressSetupTime      = 0x03; //max. 0x0f
	nst.FSMC_AddressHoldTime       = 0x00; //max. 0x0f
	nst.FSMC_DataSetupTime         = 0x04; //max. 0xff
	nst.FSMC_BusTurnAroundDuration = 0x00; //max. 0x0f
	nst.FSMC_CLKDivision           = 0x00; //max. 0x0f
	nst.FSMC_DataLatency           = 0x00; //not used
	nst.FSMC_AccessMode            = FSMC_AccessMode_A;

	FSMC_NORSRAMInitTypeDef nsi;
	nsi.FSMC_Bank 					= FSMC_Bank1_NORSRAM2;
	nsi.FSMC_DataAddressMux 		= FSMC_DataAddressMux_Disable;
	nsi.FSMC_MemoryType 			= FSMC_MemoryType_SRAM;
	nsi.FSMC_MemoryDataWidth 		= FSMC_MemoryDataWidth_16b;
	nsi.FSMC_BurstAccessMode 		= FSMC_BurstAccessMode_Disable;
	nsi.FSMC_AsynchronousWait 		= FSMC_AsynchronousWait_Disable;
	nsi.FSMC_WaitSignalPolarity 	= FSMC_WaitSignalPolarity_Low;
	nsi.FSMC_WrapMode 				= FSMC_WrapMode_Disable;
	nsi.FSMC_WaitSignalActive 		= FSMC_WaitSignalActive_BeforeWaitState;
	nsi.FSMC_WriteOperation 		= FSMC_WriteOperation_Enable;
	nsi.FSMC_WaitSignal 			= FSMC_WaitSignal_Disable;
	nsi.FSMC_ExtendedMode 			= FSMC_ExtendedMode_Disable;
	nsi.FSMC_WriteBurst 			= FSMC_WriteBurst_Disable;
	nsi.FSMC_ReadWriteTimingStruct 	= &nst;
	nsi.FSMC_WriteTimingStruct 		= &nst;
	FSMC_NORSRAMInit(&nsi);

	/* Enable FSMC Bank1_SRAM2 Bank */
	FSMC_NORSRAMCmd(FSMC_Bank1_NORSRAM2, ENABLE);
}

bool xmem_test()
{
	volatile uint32_t addr;

	// write
	for(addr = XMEM_START; addr < XMEM_END + 1 ; addr += 4)
	{
	    *(uint32_t*)addr = addr;
	}

	// read
	//for(addr = XMEM_START; addr < (XMEM_START + 0x8000) ; addr += 4)
	for(addr = XMEM_START; addr < XMEM_END + 1 ; addr += 4)
	{
		if(*(uint32_t *)addr != addr)
			return false;
	}

	return true;
}

